if not _G.FastNETMenu then
    _G.FastNETMenu = {}
    FastNETMenu.mod_path = ModPath
    FastNETMenu.option_path = SavePath .. "FastNETMenu.json"
    FastNETMenu.reconnect_data_path = SavePath .. "FastNETMenu_reconnect.json"
    FastNETMenu.options = {
        SortAlphabeticallyHeists = false,
        use_standard_scene = false
    }
    FastNETMenu.fast_net_core = "fast_net_core"
	FastNETMenu.fast_net = "fast_net"
	FastNETMenu.fast_net_friends = "fast_net_friends"
    FastNETMenu.fast_net_filter = "fast_net_filter"

    function FastNETMenu:Save()
        local file = io.open( self.option_path, "w+" )
        if file then
            file:write( json.encode( self.options ) )
            file:close()
        end
    end

    function FastNETMenu:Load()
        local file = io.open( self.option_path, "r" )
        if file then
			for k, v in pairs(json.decode(file:read("*all"))) do
				self.options[k] = v
			end
			file:close()
		end
    end

    function FastNETMenu:GetOption(id)
        return self.options[id]
    end

    function FastNETMenu:SaveRoom(room_id)
		local file = io.open(self.reconnect_data_path, "w+")
		if file then
			file:write(room_id)
			file:close()
		end
	end

	function FastNETMenu:Reconnect()
        local room_id
		local file = io.open(self.reconnect_data_path, "r")
		if file then
			room_id = file:read("*all")
			file:close()
		end
		if room_id then
			managers.network.matchmake:join_server(room_id)
		else

            local dialog_data = {
                title = managers.localization:text("dialog_warning_title"),
                text = managers.localization:text("wolfhud_no_lobby_collect_to")
            }

            local no_button = {
                text = managers.localization:text("dialog_ok"),
                cancel_button = true
            }
            dialog_data.button_list = {
                no_button
            }

            managers.system_menu:show(dialog_data)
        end
    end

    FastNETMenu:Load()
end

    function FastNETMenu:apply_lobby_filter(diff_filter)
        if diff_filter > 8 then
            managers.network.matchmake:add_lobby_filter("difficulty", diff_filter - 6, "equalto_or_greater_than")
        end
    end

    function FastNETMenu:fix_preset_difficulties(presets)
        for _, preset in pairs(presets or {}) do
            if preset.difficulty_id > 8 then
                local min_difficulty = preset.difficulty_id - 6
                preset.difficulty_id = math.round(min_difficulty + math.rand(8 - min_difficulty))
                preset.difficulty = tweak_data:index_to_difficulty(preset.difficulty_id)
            end
        end
    end

    Hooks:Add("LocalizationManagerPostInit", "LocalizationManagerPostInit_FastNETMenu", function( loc )
        for _, filename in pairs(file.GetFiles( FastNETMenu.mod_path .. "Localization/")) do
            local str = filename:match('^(.*).json$')
            if str and Idstring(str) and Idstring(str):key() == SystemInfo:language():key() then
                loc:load_localization_file( FastNETMenu.mod_path .. "Localization/" .. filename)
                break
            end
        end
        loc:load_localization_file( FastNETMenu.mod_path .. "Localization/english.json", false)
    end)

    Hooks:Add("MenuManagerSetupCustomMenus", "MenuManagerSetupCustomMenus_FastNetStandalone", function( menu_manager, nodes )

        MenuHelper:NewMenu( FastNETMenu.fast_net_core )
        MenuHelper:NewMenu( FastNETMenu.fast_net )
        MenuHelper:NewMenu( FastNETMenu.fast_net_friends )
    end)

    Hooks:Add( "MenuManagerInitialize", "MenuManagerInitialize_FastNETMenu", function( menu_manager )

        MenuCallbackHandler.callback_SortAlphabeticallyHeists = function(self, item)
            FastNETMenu.options.SortAlphabeticallyHeists = (item:value() == "on" and true or false)
            FastNETMenu:Save()
        end

        MenuCallbackHandler.callback_use_standard_scene = function(self, item)
            FastNETMenu.options.use_standard_scene = (item:value() == "on" and true or false)
            FastNETMenu:Save()
        end

        FastNETMenu:Load()

        MenuHelper:LoadFromJsonFile( FastNETMenu.mod_path .. "Options/options.json", FastNETMenu, FastNETMenu.options )
    end )

    Hooks:Add("MenuManagerPopulateCustomMenus", "MenuManagerPopulateCustomMenus_FastNetStandalone", function(menu_manager, nodes)

        -- Fast.Net Buttons
        MenuHelper:AddButton({
            id = "play_STEAM_online",
            title = "menu_play_online",
            desc = "menu_play_online_help",
            callback = "find_online_games",
            next_node = FastNETMenu.fast_net,
            menu_id = FastNETMenu.fast_net_core
        })

        MenuHelper:AddButton({
            id = "play_STEAM_online_with_friends",
            title = "menu_play_with_friends",
            desc = "menu_play_with_friends_help",
            callback = "find_online_games_with_friends",
            next_node = FastNETMenu.fast_net_friends,
            menu_id = FastNETMenu.fast_net_core
        })
    end)

    Hooks:Add("MenuManagerBuildCustomMenus", "MenuManagerBuildCustomMenus_FastNetStandalone", function(menu_manager, nodes)
        local node_main = nodes.main

        nodes[FastNETMenu.fast_net_core] = MenuHelper:BuildMenu(FastNETMenu.fast_net_core)
        if node_main then
            MenuHelper:AddMenuItem(nodes.main, FastNETMenu.fast_net_core, "fastnet_sa", "menu_fastnet_help", "crimenet", "before")
        end

        nodes[FastNETMenu.fast_net] = MenuHelper:BuildMenu(FastNETMenu.fast_net)
        FastNETMenu.fast_net_node = nodes[FastNETMenu.fast_net]

        nodes[FastNETMenu.fast_net_friends] = MenuHelper:BuildMenu(FastNETMenu.fast_net_friends)
        FastNETMenu.fast_net_friends_node = nodes[FastNETMenu.fast_net_friends]

        local isWin32 = blt.blt_info().platform == "mswindows"
        local use_standard = FastNETMenu:GetOption("use_standard_scene")
        local scene_background

        if use_standard and isWin32 then
            scene_background = "standard"
        elseif isWin32 then
            scene_background = "safe"
        else
            scene_background = "standard"
        end

        local arguments = {
			_meta = "node",
			[1] = {
				_meta = "legend",
				name = "menu_legend_select"
			},
			[2] = {
				_meta = "legend",
				name = "wolfhud_fastnet_update",
				pc = "true"
			},
			[3] = {
				_meta = "legend",
				name = "wolfhud_fastnet_filters",
				pc = "true"
			},
            [4] = {
				_meta = "legend",
				name = "wolfhud_fastnet_reconnect",
				pc = "true"
			},
			[5] = {
				_meta = "legend",
				name = "menu_legend_back"
			},
			--align_line = 0.5,
			back_callback = "stop_multiplayer",
			gui_class = "MenuNodeTableGui",
			legend = {
				_meta = "legend",
				name = "menu_legend_back",
				_meta = "legend",
				name = "menu_legend_select",
				_meta = "legend",
				name = "wolfhud_fastnet_update",
				_meta = "legend",
				name = "wolfhud_fastnet_filters",
                _meta = "legend",
                name = "wolfhud_fastnet_reconnect"
			},
            menu_components = "",
            modifier = "MenuSTEAMHostBrowser",
            name = FastNETMenu.fast_net,
            refresh = "MenuSTEAMHostBrowser",
            stencil_align = "right",
            stencil_image = "bg_creategame",
            topic_id = "menu_play_online",
            type = "MenuNodeServerList",
            update = "MenuSTEAMHostBrowser",
            scene_state = scene_background
        }
        local type = "MenuNodeServerList"
        if type then
            node_class = CoreSerialize.string_to_classtable(type)
        end
        nodes[FastNETMenu.fast_net] = node_class:new(arguments)

        local callback_handler = CoreSerialize.string_to_classtable("MenuCallbackHandler")
        nodes[FastNETMenu.fast_net]:set_callback_handler(callback_handler:new())

        local arugements_friends = {
			_meta = "node",
			[1] = {
				_meta = "legend",
				name = "menu_legend_select"
			},
			[2] = {
				_meta = "legend",
				name = "wolfhud_fastnet_update",
				pc = "true"
			},
			[3] = {
				_meta = "legend",
				name = "menu_legend_back"
			},
			--align_line = 0.5,
			back_callback = "stop_multiplayer",
			gui_class = "MenuNodeTableGui",
			legend = {
				_meta = "legend",
				name = "menu_legend_back",
				_meta = "legend",
				name = "menu_legend_select",
				_meta = "legend",
				name = "menu_legend_update",
                _meta = "legend"
			},
			menu_components = "",
            modifier = "MenuSTEAMHostBrowser",
            name = FastNETMenu.fast_net_friends,
            refresh = "MenuSTEAMHostBrowser",
            stencil_align = "right",
            stencil_image = "bg_creategame",
            topic_id = "menu_play_online",
            type = "MenuNodeServerList",
            update = "MenuSTEAMHostBrowser",
            scene_state = scene_background
        }
        local type = "MenuNodeServerList"
        if type then
            node_class = CoreSerialize.string_to_classtable(type)
        end
        nodes[FastNETMenu.fast_net_friends] = node_class:new(arugements_friends)

        local callback_handler = CoreSerialize.string_to_classtable("MenuCallbackHandler")
        nodes[FastNETMenu.fast_net_friends]:set_callback_handler(callback_handler:new())
    end)

if MenuManager then

    function MenuCallbackHandler:save_crimenet_filters()
        managers.savefile:save_setting(true)
        managers.network.matchmake:search_lobby(managers.network.matchmake:search_friends_only())
    end

    function MenuCallbackHandler:load_filters()
        if managers.network.matchmake and managers.network.matchmake.load_user_filters then
            managers.network.matchmake:load_user_filters()
            managers.network.matchmake:search_lobby(managers.network.matchmake:search_friends_only())
        end
    end

    function MenuSTEAMHostBrowser:add_filter(node)
    end

    local _find_online_games_original = MenuCallbackHandler._find_online_games

    function MenuCallbackHandler:_find_online_games(friends_only)
        MenuCallbackHandler:play_online_game()
        _find_online_games_original(self, friends_only)
	end

    function MenuCallbackHandler:chocie_one_down_filter(item)
        local allow_one_down = item:value() == "on" and true or false
        Global.game_settings.search_one_down_lobbies = allow_one_down

        managers.user:set_setting("crimenet_filter_one_down", allow_one_down)

        managers.network.matchmake:search_lobby(managers.network.matchmake:search_friends_only())
    end

    function MenuCallbackHandler:setup_join_cs_manager(item, ...)
        local params = item:parameters()
        if params.is_crime_spree then
            managers.crime_spree:join_server(params)
        end
    end

    function MenuSTEAMHostBrowser:refresh_node(node, info, friends_only)

            -- if not FastNETMenu.Version then
                -- managers.network.matchmake:reset_crash_fix()
                -- managers.network.matchmake:search_lobby(managers.network.matchmake:search_friends_only())
            -- end

            local new_node = node

            if not info then
                managers.menu:add_back_button(new_node)
                return new_node
            end

            local room_list = info.room_list
            local attribute_list = info.attribute_list

            local dead_list = {}
            for _, item in ipairs(node:items()) do
                if not item:parameters().back and not item:parameters().filter and not item:parameters().pd2_corner then
                    dead_list[item:parameters().room_id] = true
                end
            end

            for i, room in ipairs(room_list) do
                if managers.network.matchmake:is_server_ok(friends_only, room, attribute_list[i], nil) then
                    local host_name = tostring(room.owner_name)
                    local attributes_numbers = attribute_list[i].numbers
                    local attributes_mutators = attribute_list[i].mutators
                    if attributes_numbers then
                        dead_list[room.room_id] = nil
                        local level_index, job_index = managers.network.matchmake:_split_attribute_number(attributes_numbers[1], 1000)
                        local level_id = tweak_data.levels:get_level_name_from_index(level_index)
                        local name_id = level_id and tweak_data.levels[level_id] and tweak_data.levels[level_id].name_id
                        local level_name = name_id and managers.localization:text(name_id) or "LEVEL NAME ERROR"
                        local job_id = tweak_data.narrative:get_job_name_from_index(math.floor(job_index))
                        local job_name = job_id and tweak_data.narrative.jobs[job_id] and managers.localization:text(tweak_data.narrative.jobs[job_id].name_id) or "CONTRACTLESS"
                        local job_days = job_id and (tweak_data.narrative.jobs[job_id].job_wrapper and  table.maxn(tweak_data.narrative.jobs[tweak_data.narrative.jobs[job_id].job_wrapper[1]].chain) or table.maxn(tweak_data.narrative.jobs[job_id].chain)) or 1
                        local is_pro = job_id and (tweak_data.narrative.jobs[job_id].professional and tweak_data.narrative.jobs[job_id].professional or false) or false
                        local difficulty_num = attributes_numbers[2]
                        local difficulty = tweak_data:index_to_difficulty(difficulty_num) or "error"
                        local is_one_down = (tonumber(attribute_list[i].one_down) or 0) == 1
                        local state_string_id = tweak_data:index_to_server_state(attributes_numbers[4])
                        local state_name = state_string_id and managers.localization:text("menu_lobby_server_state_" .. state_string_id) or "UNKNOWN"
                        local display_job = job_name .. ((job_name ~= level_name and job_name ~= "CONTRACTLESS" and level_name ~= "CONTRACTLESS" and job_days > 1) and " (" .. level_name .. ")" or "")
                        local state = attributes_numbers[4]
                        local num_plrs = attributes_numbers[5]
                        local kick_option = attributes_numbers[8]
                        local kick_suffix = {[1] = "server", [2] = "vote", [0] = "disabled"}
                        local kick_option_name = "menu_kick_" .. (kick_suffix[kick_option] or "error")
                        local job_plan = attributes_numbers[10]
                        local job_plan_suffix = {"plan_loud", "plan_stealth"}
                        local job_plan_name = "menu_" .. (job_plan_suffix[job_plan] or "any")
                        local attribute_crimespree = attribute_list[i].crime_spree
                        local is_crime_spree = attribute_crimespree and 0 <= attribute_crimespree
                        local crime_spree_mission = attribute_list[i].crime_spree_mission
                        local crime_spree_mission_name = "CONTRACTLESS"
                        local mods = attribute_list[i].mods
                        local is_skirmish = attribute_list[i].skirmish and attribute_list[i].skirmish > 0
                        local skirmish = attribute_list[i].skirmish
                        local skirmish_wave = attribute_list[i].skirmish_wave
                        if crime_spree_mission then
                            local mission_data = managers.crime_spree:get_mission(crime_spree_mission)
                            if mission_data then
                                local tweak = tweak_data.levels[mission_data.level.level_id]
                                crime_spree_mission_name = managers.localization:text(tweak and tweak.name_id or "UNKNOWN")
                            end
                        end
                        local item = new_node:item(room.room_id)
                        if not item and not (state  ~= 1 and not tweak_data.narrative.jobs[job_id]) then
                            print("ADD", name_str)
                            local params = {
                                name = room.room_id,
                                text_id = name_str,
                                room_id = room.room_id,
                                columns = {
                                    utf8.to_upper(host_name),
                                    utf8.to_upper(is_crime_spree and crime_spree_mission_name or display_job),
                                    utf8.to_upper(state_name),
                                    tostring(num_plrs) .. "/4 ",
                                    (job_plan == 1 and utf8.char(57364) or job_plan == 2 and utf8.char(57363) or "")
                                },
                                pro = is_pro,
                                days = job_days,
                                level_name = job_id,
                                real_level_name = display_job,
                                level_id = level_id,
                                state_name = state_name,
                                difficulty = difficulty,
                                job_plan = job_plan,
                                job_plan_name = job_plan_name,
                                difficulty_num = difficulty_num or 2,
                                is_one_down = is_one_down,
                                host_name = host_name,
                                state = state,
                                num_plrs = num_plrs,
                                kick_option = kick_option,
                                kick_option_name = kick_option_name,
                                friend = is_friend,
                                is_crime_spree = is_crime_spree,
                                crime_spree = attribute_crimespree,
                                crime_spree_mission = crime_spree_mission,
                                crime_spree_mission_name = crime_spree_mission_name,
                                mutators = attributes_mutators,
                                is_skirmish = skirmish and skirmish > 0,
                                skirmish = skirmish,
                                skirmish_wave = skirmish_wave,
                                mods = mods,
                                callback = "setup_join_cs_manager connect_to_lobby",
                                localize = false
                            }
                            local new_item = new_node:create_item({
                                type = "ItemServerColumn"
                            }, params)
                            new_node:add_item(new_item)
                        elseif not (state  ~= 1 and not tweak_data.narrative.jobs[job_id]) then
                            if item:parameters().real_level_name ~= display_job then
                                item:parameters().columns[2] = utf8.to_upper(display_job)
                                item:parameters().level_name = job_id
                                item:parameters().level_id = level_id
                                item:parameters().real_level_name = display_job
                            end
                            if item:parameters().state ~= state then
                                item:parameters().columns[3] = state_name
                                item:parameters().state = state
                                item:parameters().state_name = state_name
                            end
                            if item:parameters().difficulty ~= difficulty then
                                item:parameters().difficulty = difficulty
                            end
                            if item:parameters().room_id ~= room.room_id then
                                item:parameters().room_id = room.room_id
                            end
                            if item:parameters().num_plrs ~= num_plrs then
                                item:parameters().num_plrs = num_plrs
                                item:parameters().columns[4] = tostring(num_plrs) .. "/4 "
                            end
                            if item:parameters().crime_spree ~= attribute_crimespree then
                                item:parameters().is_crime_spree = is_crime_spree
                                item:parameters().crime_spree = attribute_crimespree
                            end
                            if item:parameters().crime_spree_mission ~= crime_spree_mission then
                                item:parameters().crime_spree_mission = crime_spree_mission
                                item:parameters().crime_spree_mission_name = crime_spree_mission_name
                            end
                            if item:parameters().difficulty_num ~= difficulty_num then
                                item:parameters().difficulty_num = difficulty_num
                            end
                            if item:parameters().is_one_down ~= is_one_down then
                                item:parameters().is_one_down = is_one_down
                            end
                            if item:parameters().mutators ~= attributes_mutators then
                                item:parameters().mutators = attributes_mutators
                            end
                            if item:parameters().job_plan ~= job_plan then
                                item:parameters().job_plan = job_plan
                                item:parameters().job_plan_name = job_plan_name
                                item:parameters().columns[5] = (job_plan == 1 and utf8.char(57364) or job_plan == 2 and utf8.char(57363) or "")
                            end
                        elseif item then
                            new_node:delete_item(room.room_id)
                        end
                    end
                end
            end
            for name, _ in pairs(dead_list) do
                new_node:delete_item(name)
            end
            table.sort(new_node:items(), function (a, b)
                local a_diff = (a:parameters().is_crime_spree and a:parameters().crime_spree or a:parameters().difficulty_num or 2) + (a:parameters().is_one_down and 0.5 or 0)
                local b_diff = (b:parameters().is_crime_spree and b:parameters().crime_spree or b:parameters().difficulty_num or 2) + (b:parameters().is_one_down and 0.5 or 0)
                local lower_difficulty 	= (a_diff < b_diff)
                local equal_difficulty 	= (a_diff == b_diff)
                local less_players 		= (a:parameters().num_plrs or 0) < (b:parameters().num_plrs or 0)
                return lower_difficulty or (equal_difficulty and less_players) or false
            end)

            managers.menu:add_back_button(new_node)
            return new_node
        end

    local modify_filter_node_actual = MenuCrimeNetFiltersInitiator.modify_node
	local choice_difficulty_filter_original = MenuCallbackHandler.choice_difficulty_filter
    local add_filters_original = MenuCrimeNetFiltersInitiator.add_filters

	local server_count = {10, 20, 30, 40, 50, 60, 70}
	local difficulties = {
		{ name_id = "menu_any", 					value = -1 },
		{ name_id = "menu_difficulty_normal", 		value = 2 },
		{ name_id = "menu_difficulty_hard", 		value = 3 },
		{ name_id = "menu_difficulty_very_hard", 	value = 4 },
		{ name_id = "menu_difficulty_overkill", 	value = 5 },
		{ name_id = "menu_difficulty_easy_wish", 	value = 6 },
		{ name_id = "menu_difficulty_apocalypse", 	value = 7 },
		{ name_id = "menu_difficulty_sm_wish", 		value = 8 },
		{ name_id = "menu_difficulty_hard", 		value = 9, 	suffix = "+" },
		{ name_id = "menu_difficulty_very_hard", 	value = 10, suffix = "+" },
		{ name_id = "menu_difficulty_overkill", 	value = 11, suffix = "+" },
		{ name_id = "menu_difficulty_easy_wish", 	value = 12, suffix = "+" },
		{ name_id = "menu_difficulty_apocalypse", 	value = 13, suffix = "+" },
	}

	function MenuCrimeNetFiltersInitiator:modify_node(original_node, ...)
		local new_node = modify_filter_node_actual(self, original_node, ...)
		if server_count ~= nil then
			local max_lobbies = new_node:item("max_lobbies_filter")
			if max_lobbies ~= nil then
				max_lobbies:clear_options()
				for _, count in ipairs(server_count) do
					max_lobbies:add_option(CoreMenuItemOption.ItemOption:new({
						_meta = "option",
						text_id = tostring(count),
						value = count,
						localize = false
					}))
				end
				max_lobbies:_show_options(nil)
			end
		end

		if difficulties ~= nil then
			local diff_filter = new_node:item("difficulty_filter")
			if diff_filter ~= nil then
				diff_filter:clear_options()
				for _, v in ipairs(difficulties) do
					diff_filter:add_option(CoreMenuItemOption.ItemOption:new({
						_meta = "option",
						text_id = managers.localization:text(v.name_id) .. (v.suffix or ""),
						value = v.value,
						localize = false
					}))
				end
				diff_filter:_show_options(nil)
				local matchmake_filters = managers.network.matchmake:lobby_filters()
				if matchmake_filters and matchmake_filters.difficulty then
					diff_filter:set_value(matchmake_filters.difficulty.value + (matchmake_filters.difficulty.comparision_type == "equal" and 0 or 6))
				end
			end
		end

		return new_node
	end

	function MenuCallbackHandler:choice_difficulty_filter(item, ...)
		choice_difficulty_filter_original(self, item, ...)
		FastNETMenu:apply_lobby_filter(item:value())
	end

    function MenuCrimeNetFiltersInitiator:add_filters(node)
        add_filters_original(self, node)

        if FastNETMenu:GetOption("SortAlphabeticallyHeists") then
            table.sort(node:item("job_id_filter")._all_options, function(a, b)
                if b:parameters().text_id ~= "menu_any" then
                    return a:parameters().text_id < b:parameters().text_id
                end
            end)
        end
    end

end

if MenuNodeGui then
	local _create_legends_original = MenuNodeGui._create_legends
	function MenuNodeGui._create_legends(self, node)
		_create_legends_original(self, node)
		local is_pc = managers.menu:is_pc_controller()
		if is_pc then
			local has_pc_legend = false
			local visible_callback_name, visible_callback
			local t_text = ""
			for i, legend in pairs(node:legends()) do
				visible_callback_name = legend.visible_callback
				visible_callback = nil
				if visible_callback_name then
					visible_callback = callback(node.callback_handler, node.callback_handler, visible_callback_name)
				end
				if (not is_pc or legend.pc) and (not visible_callback or visible_callback(self)) then
					has_pc_legend = has_pc_legend or legend.pc
					local spacing = i > 1 and "  " or ""
					t_text = t_text .. spacing .. utf8.to_upper(managers.localization:text(legend.string_id, {
						BTN_X = managers.localization:btn_macro("menu_update") or managers.localization:get_default_macro("BTN_X"),
						BTN_Y = managers.localization:btn_macro("menu_toggle_filters") or managers.localization:get_default_macro("BTN_Y"),
						BTN_START = managers.localization:btn_macro("menu_respec_tree_all") or managers.localization:get_default_macro("BTN_START"),
						BTN_BACK = managers.localization:btn_macro("back")
					}))
				end
			end
			local text = self._legends_panel:child(0)
			text:set_text(t_text)
			local _, _, w, h = text:text_rect()
			text:set_size(w, h)
			self:_layout_legends()
		else
			local has_pc_legend = false
			local visible_callback_name, visible_callback
			local t_text = ""
			for i, legend in pairs(node:legends()) do
				visible_callback_name = legend.visible_callback
				visible_callback = nil
				if visible_callback_name then
					visible_callback = callback(node.callback_handler, node.callback_handler, visible_callback_name)
				end
				if (not is_pc or legend.pc) and (not visible_callback or visible_callback(self)) then
					has_pc_legend = has_pc_legend or legend.pc
					local spacing = i > 1 and "  " or ""
					t_text = t_text .. spacing .. utf8.to_upper(managers.localization:text(legend.string_id, {
						BTN_X = managers.localization:btn_macro("menu_toggle_legends") or managers.localization:get_default_macro("BTN_X"),
						BTN_Y = managers.localization:btn_macro("menu_update") or managers.localization:get_default_macro("BTN_Y"),
						BTN_START = managers.localization:btn_macro("menu_respec_tree_all") or managers.localization:get_default_macro("BTN_START"),
						BTN_BACK = managers.localization:btn_macro("back")
					}))
				end
			end
			local text = self._legends_panel:child(0)
			text:set_text(t_text)
			local _, _, w, h = text:text_rect()
			text:set_size(w, h)
			self:_layout_legends()
		end
	end
end

if CrimeNetManager then
    local _setup_original = CrimeNetManager._setup
    local update_difficulty_filter_original = CrimeNetManager.update_difficulty_filter

    function CrimeNetManager:_setup(...)
        _setup_original(self, ...)
        FastNETMenu:fix_preset_difficulties(self._presets)
    end

    function CrimeNetManager:update_difficulty_filter(...)
        update_difficulty_filter_original(self, ...)
        FastNETMenu:fix_preset_difficulties(self._presets)
    end
end

if NetworkMatchMakingSTEAM then
    Hooks:PostHook(NetworkMatchMakingSTEAM, "join_server", "FastNet_Reconnect_Steam", function(self, room_id)
        FastNETMenu:SaveRoom(room_id)
		log("[Reconnect to server] Saving Steam room ID "..room_id)
	end)

    local load_user_filters_original = NetworkMatchMakingSTEAM.load_user_filters

	function NetworkMatchMakingSTEAM:load_user_filters(...)
		load_user_filters_original(self, ...)
		FastNETMenu:apply_lobby_filter(managers.user:get_setting("crimenet_filter_difficulty"))
	end
end

if NetworkMatchMakingEPIC then
    Hooks:PostHook(NetworkMatchMakingEPIC, "join_server", "FastNet_Reconnect_Epic", function(self, room_id)
		FastNETMenu:SaveRoom(room_id)
		log("[Reconnect to server] Saving Epic room ID "..room_id)
	end)

    local load_user_filters_original = NetworkMatchMakingEPIC.load_user_filters

	function NetworkMatchMakingEPIC:load_user_filters(...)
		load_user_filters_original(self, ...)
		FastNETMenu:apply_lobby_filter(managers.user:get_setting("crimenet_filter_difficulty"))
	end
end