function MenuNodeTableGui:_setup_panels(node)
	MenuNodeTableGui.super._setup_panels(self, node)
	local safe_rect_pixels = self:_scaled_size()
	local mini_info = self.safe_rect_panel:panel({
		x = 0,
		y = self._info_bg_rect:h(),
		w = self._info_bg_rect:w(),
		h = self.safe_rect_panel:h() - self._info_bg_rect:h()
	})
	local mini_text = mini_info:text({
		x = self.safe_rect_panel:w() - tweak_data.menu.info_padding * 12,
		y = tweak_data.menu.info_padding,
		w = tweak_data.menu.info_padding * 11,
		h = 35,
		align = "left",
		halign = "top",
		vertical = "top",
		font = tweak_data.menu.pd2_small_font,
		font_size = tweak_data.menu.pd2_small_font_size + 2,
		color = Color.white or PDTH_Menu and Color.yellow,
		layer = self.layers.items,
		text = ""
	})
	mini_info:set_width(self._info_bg_rect:w() - tweak_data.menu.info_padding * 38)
	mini_text:set_width(mini_info:w())
	mini_info:set_height(35)
	mini_text:set_height(35)
	mini_info:set_top(self._info_bg_rect:bottom() + tweak_data.menu.info_padding - 12)
	mini_text:set_top(0)
	mini_info:set_left(tweak_data.menu.info_padding)
	mini_text:set_left(0)
	self._mini_info_text = mini_text
end


function MenuNodeTableGui:set_mini_info(text)
	self._mini_info_text:set_text(text)
end

function MenuNodeTableGui:completed_job(job_id, difficulty, require_one_down)
	local job_stats = managers.statistics._global.sessions.jobs
	local tweak_jobs = tweak_data.narrative.jobs
	local job_wrapper = nil

	if tweak_data.narrative:has_job_wrapper(job_id) then
		job_wrapper = tweak_jobs[job_id].job_wrapper
	elseif tweak_data.narrative:is_wrapped_to_job(job_id) then
		job_wrapper = tweak_jobs[tweak_jobs[job_id].wrapped_to_job].job_wrapper
	end

	local function single_job_count(job_id, difficulty, require_one_down)
		local stat_prefix = tostring(job_id) .. "_" .. tostring(difficulty)
		local stat_suffix = "_completed"
		local count = 0
		count = count + (job_stats[stat_prefix .. "_od" .. stat_suffix] or 0)

		if not require_one_down then
			count = count + (job_stats[stat_prefix .. stat_suffix] or 0)
		end

		return count
	end

	local count = 0

	if job_wrapper then
		local count = 0

		for _, wrapped_job in ipairs(job_wrapper) do
			count = count + single_job_count(wrapped_job, difficulty, require_one_down)
		end

		return count
	end

	return single_job_count(job_id, difficulty, require_one_down)
end

function MenuNodeTableGui:_create_menu_item(row_item)
	if row_item.type == "column" then
		local columns = row_item.node:columns()
		local total_proportions = row_item.node:parameters().total_proportions
		row_item.gui_panel = self.item_panel:panel({
			x = self:_right_align(),
			w = self.item_panel:w()
		})
		row_item.gui_columns = {}
		local x = 0
		for i, data in ipairs(columns) do
			local text = row_item.gui_panel:text({
				font_size = self.font_size,
				x = row_item.position.x,
				y = 0,
				align = data.align,
				halign = data.align,
				vertical = "center",
				font = row_item.font,
				color = row_item.color,
				layer = self.layers.items,
				text = row_item.item:parameters().columns[i]
			})
			row_item.gui_columns[i] = text
			local _, _, w, h = text:text_rect()
			text:set_h(h)
			local w = data.proportions / total_proportions * row_item.gui_panel:w()
			text:set_w(w)
			text:set_x(x)
			x = x + w
		end
		local x, y, w, h = row_item.gui_columns[1]:text_rect()
		row_item.gui_panel:set_height(h)
	elseif row_item.type == "server_column" then
		--row_item.font = tweak_data.menu.pd2_medium_font_id
		local columns = row_item.node:columns()
		local total_proportions = row_item.node:parameters().total_proportions
		local safe_rect = self:_scaled_size()
		local xl_pad = 80
		row_item.gui_panel = self.item_panel:panel({
			x = safe_rect.width / 2 - xl_pad,
			w = safe_rect.width / 2 + xl_pad - tweak_data.menu.info_padding
		})
		row_item.gui_columns = {}
		local x = 0
		for i, data in ipairs(columns) do
			local color = row_item.color
			if i == 1 and row_item.item:parameters().friend then
				color = tweak_data.screen_colors.friend_color
			elseif i == 2 and row_item.item:parameters().pro then
				color = tweak_data.screen_colors.pro_color
			elseif row_item.item:parameters().mutators then
				color = tweak_data.screen_colors.mutators_color
			end
			
			local new_font_size
			if PDTH_Menu then
				new_font_size = tweak_data.menu.server_list_font_size
			else
				new_font_size = math.round(row_item.font_size * 0.77)
			end
			local text = row_item.gui_panel:text({
				x = row_item.position.x,
				y = 0,
				align = data.align,
				halign = data.align,
				vertical = "center",
				font = row_item.font,
				font_size = new_font_size,
				color = color,
				layer = self.layers.items,
				text = row_item.item:parameters().columns[i]
			})
			row_item.gui_columns[i] = text
			local _, _, w, h = text:text_rect()
			text:set_h(h)
			local w = data.proportions / total_proportions * row_item.gui_panel:w()
			text:set_w(w + (i == 2 and 10 or 0))
			text:set_x(x)
			x = x + w
		end
		local x, y, w, h = row_item.gui_columns[1]:text_rect()
		row_item.gui_panel:set_height(h)	
		
		local x = row_item.gui_columns[2]:right()
		local y = 0
		row_item.difficulty_icons = {}
		if row_item.item:parameters().is_crime_spree then
			local spree_level = row_item.gui_panel:text({
				font_size = tweak_data.menu.server_list_font_size,
				x = x,
				y = y,
				w = 60,
				h = h,
				align = "right",
				halign = "center",
				vertical = "center",
				font = row_item.font,
				font_size = math.round(row_item.font_size * 0.77),
				color = tweak_data.screen_colors.crime_spree_risk,
				layer = self.layers.items,
				text = managers.experience:cash_string(tonumber(row_item.item:parameters().crime_spree), "") .. managers.localization:get_default_macro("BTN_SPREE_TICKET"),
			})
			table.insert(row_item.difficulty_icons, spree_level)
		elseif row_item.item:parameters().is_skirmish then
			local wave = row_item.item:parameters().skirmish_wave
			local text = managers.localization:to_upper_text("menu_skirmish_wave_number", {
				wave = wave
			})
			local skirmish_wave = row_item.gui_panel:text({
				font_size = tweak_data.menu.server_list_font_size,
				x = x,
				y = y,
				w = 60,
				h = h,
				align = "right",
				halign = "center",
				vertical = "center",
				font = row_item.font,
				font_size = math.round(row_item.font_size * 0.77),
				layer = self.layers.items,
				text = text,
				color = tweak_data.screen_colors.skirmish_color,
				font = tweak_data.menu.pd2_small_font,
				font_size = tweak_data.menu.server_list_font_size
			})
		else
			local difficulty_stars = row_item.item:parameters().difficulty_num
			local start_difficulty = 3
			local num_difficulties = 6
			local spacing = 14
			for i = start_difficulty, difficulty_stars do
				local difficulty_id = tweak_data:index_to_difficulty(i)
				local skull_texture = difficulty_id and tweak_data.gui.blackscreen_risk_textures[difficulty_id] or "guis/textures/pd2/risklevel_blackscreen"
				local skull = row_item.gui_panel:bitmap({
					texture = skull_texture,
					x = x,
					y = y,
					w = h,
					h = h,
					--blend_mode = "add",
					layer = self.layers.items,
					color = tweak_data.screen_colors.risk
				})
				x = x + (spacing)
				row_item.difficulty_icons[i] = skull
				--num_stars = num_stars + 1
				--skull:set_center_y(row_item.gui_columns[2]:center_y())
				if row_item.item:parameters().is_one_down then
					skull:set_color(tweak_data.screen_colors.one_down)
					-- row_item.one_down_icon = row_item.gui_panel:bitmap({
					-- 	texture = "guis/textures/pd2/cn_mini_onedown",
					-- 	x = x,
					-- 	y = y,
					-- 	w = h,
					-- 	h = h,
					-- 	--blend_mode = "add",
					-- 	layer = self.layers.items,
					-- 	color = tweak_data.screen_colors.one_down,
					-- })
				end
			end
		end
		
		local level_id = row_item.item:parameters().level_id
		local mutators = row_item.item:parameters().mutators or {}
		local mutators_list = {}
		local mutators_text = ""
		if mutators then
			managers.mutators:set_crimenet_lobby_data(mutators)
			for mutator_id, mutator_data in pairs(mutators) do
				local mutator = managers.mutators:get_mutator_from_id(mutator_id)
				if mutator then
					table.insert(mutators_list, mutator:name()) 
				end
			end
			managers.mutators:set_crimenet_lobby_data(nil)
			table.sort(mutators_list, function(a, b) 
				return a < b
			end)
			for i, mutator in ipairs(mutators_list) do
				mutators_text = string.format("%s%s", mutators_text, (mutator .. (i < #mutators_list and "\n" or "")))
			end
		end

		row_item.gui_info_panel = self.safe_rect_panel:panel({
			visible = false,
			layer = self.layers.items,
			x = 0,
			y = 0,
			w = self:_left_align(),
			h = self._item_panel_parent:h()
		})
		row_item.heist_name = row_item.gui_info_panel:text({
			visible = false,
			text = utf8.to_upper(row_item.item:parameters().level_name),
			layer = self.layers.items,
			font = self.font,
			font_size = tweak_data.menu.challenges_font_size,
			color = row_item.color,
			align = "left",
			vertical = "left"
		})
		local briefing_text = level_id and managers.localization:text(tweak_data.levels[level_id].briefing_id) or ""
		row_item.heist_briefing = row_item.gui_info_panel:text({
			visible = true,
			x = 0,
			y = 0,
			align = "left",
			halign = "top",
			vertical = "top",
			font = tweak_data.menu.pd2_small_font,
			font_size = tweak_data.menu.pd2_small_font_size,
			color = Color.white,
			layer = self.layers.items,
			text = briefing_text,
			wrap = true,
			word_wrap = true
		})
		
		local font_size = tweak_data.menu.pd2_small_font_size
		row_item.server_title = row_item.gui_info_panel:text({
			name = "server_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_server_title")) .. " ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.server_text = row_item.gui_info_panel:text({
			name = "server_text",
			text = utf8.to_upper(row_item.item:parameters().host_name),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.server_info_title = row_item.gui_info_panel:text({
			name = "server_info_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_server_state_title")) .. " ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.server_info_text = row_item.gui_info_panel:text({
			name = "server_info_text",
			text = utf8.to_upper(row_item.item:parameters().state_name) .. " " .. tostring(row_item.item:parameters().num_plrs) .. "/4 ",
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.level_title = row_item.gui_info_panel:text({
			name = "level_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_campaign_title")) .. " ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.level_text = row_item.gui_info_panel:text({
			name = "level_text",
			text = utf8.to_upper(row_item.item:parameters().real_level_name) .. " ",
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.level_pro_text = row_item.gui_info_panel:text({
			name = "level_pro_text",
			text = utf8.to_upper(row_item.item:parameters().pro and "PRO JOB" or ""),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.screen_colors.pro_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.difficulty_title = row_item.gui_info_panel:text({
			name = "difficulty_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_difficulty_title")) .. " ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		local wave = row_item.item:parameters().skirmish_wave
		local skirm_text = managers.localization:to_upper_text("menu_skirmish_wave_number", {
			wave = wave
		})
		row_item.difficulty_text = row_item.gui_info_panel:text({
			name = "difficulty_text",
			text = row_item.item:parameters().is_crime_spree and (managers.experience:cash_string(tonumber(row_item.item:parameters().crime_spree), "") .. managers.localization:get_default_macro("BTN_SPREE_TICKET")) or row_item.item:parameters().is_skirmish and skirm_text or managers.localization:to_upper_text(tweak_data.difficulty_name_ids[row_item.item:parameters().difficulty]),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.one_down_text = row_item.gui_info_panel:text({
			name = "one_down_text",
			text = managers.localization:to_upper_text("menu_one_down"),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.screen_colors.one_down,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.job_plan_title = row_item.gui_info_panel:text({
			name = "job_plan_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_job_plan_title")) .. " ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})

		local difficulty_stat = row_item.item:parameters().difficulty_num
		local stat = self:completed_job( row_item.item:parameters().level_name, tweak_data:index_to_difficulty( difficulty_stat ) )
		row_item.stats_title = row_item.gui_info_panel:text({
			name = "stats_title",
			text = managers.localization:to_upper_text("menu_stat_job_new"),
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "left",
			w = 256,
			h = font_size,
			layer = 1
		})

		row_item.stats_text = row_item.gui_info_panel:text({
			name = "stats_text",
			text = managers.localization:to_upper_text("menu_stat_job_completed_new", {
				stat = tostring(stat)
			}) .. " ",
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
			

		row_item.server_mutators_text = row_item.gui_info_panel:text({
			name = "server_mutators_text",
			text = utf8.to_upper(row_item.item:parameters().mutators and "MUTATORS ENABLED" or ""),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.screen_colors.mutators_color_text,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.crime_spree_text = row_item.gui_info_panel:text({
			name = "crime_spree_text",
			text = utf8.to_upper(row_item.item:parameters().is_crime_spree and "[CRIME SPREE]" or ""),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.screen_colors.crime_spree_risk,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})

		row_item.job_plan_text = row_item.gui_info_panel:text({
			name = "job_plan_text",
			text = utf8.to_upper(managers.localization:text("menu_job_plan_" .. tostring(row_item.item:parameters().job_plan))),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})

		row_item.mutators_title = row_item.gui_info_panel:text({
			name = "mutators_title",
			text = managers.localization:to_upper_text("menu_mutators") .. ":  ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		row_item.mutators_list = row_item.gui_info_panel:text({
			name = "days_text",
			text = utf8.to_upper(mutators_text),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})

		row_item.days_title = row_item.gui_info_panel:text({
			name = "days_title",
			text = utf8.to_upper(managers.localization:text("menu_lobby_days_title")) .. "  ",
			font = tweak_data.menu.pd2_small_font,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		row_item.days_text = row_item.gui_info_panel:text({
			name = "days_text",
			text = utf8.to_upper(math.max(row_item.item:parameters().days, 1)),
			font = tweak_data.menu.pd2_small_font,
			color = tweak_data.hud.prime_color,
			font_size = font_size,
			align = "left",
			vertical = "center",
			w = 256,
			h = font_size,
			layer = 1
		})
		
		self:_align_server_column(row_item)
		local visible = row_item.item:menu_unselected_visible(self, row_item) and not row_item.item:parameters().back
		row_item.menu_unselected = self.item_panel:bitmap({
			visible = visible,
			texture = "guis/textures/menu_unselected",
			x = 0,
			y = 0,
			layer = -1
		})
		row_item.menu_unselected:set_color(row_item.item:parameters().is_expanded and Color(0.5, 0.5, 0.5) or Color.white)
		row_item.menu_unselected:hide()
	else
		MenuNodeTableGui.super._create_menu_item(self, row_item)
	end
end
function MenuNodeTableGui:_align_server_column(row_item)
	local safe_rect = self:_scaled_size()
	self:_align_item_gui_info_panel(row_item.gui_info_panel)
	local font_size = tweak_data.menu.pd2_small_font_size
	local offset = 22 * tweak_data.scale.lobby_info_offset_multiplier
	row_item.server_title:set_font_size(font_size)
	row_item.server_text:set_font_size(font_size)
	local x, y, w, h = row_item.server_title:text_rect()
	row_item.server_title:set_x(tweak_data.menu.info_padding)
	row_item.server_title:set_y(tweak_data.menu.info_padding)
	row_item.server_title:set_w(w)
	row_item.server_text:set_lefttop(row_item.server_title:righttop())
	row_item.server_text:set_w(row_item.gui_info_panel:w())
	row_item.server_text:set_position(math.round(row_item.server_text:x()), math.round(row_item.server_text:y()))
	
	
	row_item.server_info_title:set_font_size(font_size)
	row_item.server_info_text:set_font_size(font_size)
	local x, y, w, h = row_item.server_info_title:text_rect()
	row_item.server_info_title:set_x(tweak_data.menu.info_padding)
	row_item.server_info_title:set_y(tweak_data.menu.info_padding + offset)
	row_item.server_info_title:set_w(w)
	row_item.server_info_text:set_lefttop(row_item.server_info_title:righttop())
	row_item.server_info_text:set_w(row_item.gui_info_panel:w())
	row_item.server_info_text:set_position(math.round(row_item.server_info_text:x()), math.round(row_item.server_info_text:y()))
	
	row_item.server_mutators_text:set_lefttop(row_item.server_info_text:righttop())
	row_item.server_mutators_text:set_w(row_item.gui_info_panel:w())
	row_item.server_mutators_text:set_position(math.round(row_item.server_mutators_text:x()), math.round(row_item.server_mutators_text:y()))
	local _, _, w, _ = row_item.server_mutators_text:text_rect()
	row_item.server_mutators_text:set_w(w)
	
	row_item.crime_spree_text:set_lefttop(row_item.server_mutators_text:righttop())
	row_item.crime_spree_text:set_w(row_item.gui_info_panel:w())
	row_item.crime_spree_text:set_position(math.round(row_item.crime_spree_text:x()), math.round(row_item.crime_spree_text:y()))

	row_item.level_title:set_font_size(font_size)
	row_item.level_text:set_font_size(font_size)
	row_item.level_pro_text:set_font_size(font_size)
	local x, y, w, h = row_item.level_title:text_rect()
	row_item.level_title:set_x(tweak_data.menu.info_padding)
	row_item.level_title:set_y(tweak_data.menu.info_padding + offset * 2)
	row_item.level_title:set_w(w)
	local x, y, w, h = row_item.level_text:text_rect()
	row_item.level_text:set_lefttop(row_item.level_title:righttop())
	row_item.level_text:set_w(w)
	row_item.level_text:set_position(math.round(row_item.level_text:x()), math.round(row_item.level_text:y()))
	
	row_item.level_pro_text:set_lefttop(row_item.level_text:righttop())
	row_item.level_pro_text:set_w(row_item.gui_info_panel:w())
	row_item.level_pro_text:set_position(math.round(row_item.level_pro_text:x()), math.round(row_item.level_pro_text:y()))
	
	row_item.days_title:set_font_size(font_size)
	row_item.days_text:set_font_size(font_size)
	local x, y, w, h = row_item.days_title:text_rect()
	row_item.days_title:set_x(tweak_data.menu.info_padding)
	row_item.days_title:set_y(tweak_data.menu.info_padding + offset * 3)
	row_item.days_title:set_w(w)
	row_item.days_text:set_lefttop(row_item.days_title:righttop())
	row_item.days_text:set_w(row_item.gui_info_panel:w())
	row_item.days_text:set_position(math.round(row_item.days_text:x()), math.round(row_item.days_text:y()))
	
	row_item.difficulty_title:set_font_size(font_size)
	row_item.difficulty_text:set_font_size(font_size)
	local x, y, w, h = row_item.difficulty_title:text_rect()
	row_item.difficulty_title:set_x(tweak_data.menu.info_padding)
	row_item.difficulty_title:set_y(tweak_data.menu.info_padding + offset * 4)
	row_item.difficulty_title:set_w(w)
	row_item.difficulty_text:set_lefttop(row_item.difficulty_title:righttop())
	local _, _, w, _ = row_item.difficulty_text:text_rect()
	row_item.difficulty_text:set_w(w + 8)
	row_item.difficulty_text:set_position(math.round(row_item.difficulty_text:x()), math.round(row_item.difficulty_text:y()))
	
	row_item.one_down_text:set_lefttop(row_item.difficulty_text:righttop())
	row_item.one_down_text:set_w(row_item.gui_info_panel:w())
	row_item.one_down_text:set_position(math.round(row_item.one_down_text:x()), math.round(row_item.one_down_text:y()))
	row_item.one_down_text:set_visible(row_item.item:parameters().is_one_down or false)
	
	row_item.job_plan_title:set_font_size(font_size)
	row_item.job_plan_text:set_font_size(font_size)
	local x, y, w, h = row_item.job_plan_title:text_rect()
	row_item.job_plan_title:set_x(tweak_data.menu.info_padding)
	row_item.job_plan_title:set_y(tweak_data.menu.info_padding + offset * 5)
	row_item.job_plan_title:set_w(w)
	row_item.job_plan_text:set_lefttop(row_item.job_plan_title:righttop())
	row_item.job_plan_text:set_w(row_item.gui_info_panel:w())
	row_item.job_plan_text:set_position(math.round(row_item.job_plan_text:x()), math.round(row_item.job_plan_text:y()))

	local mutators_active = row_item.item:parameters().mutators or false
		
	local _, _, w, h = row_item.mutators_list:text_rect()
	row_item.mutators_list:set_w(row_item.gui_info_panel:w())
	row_item.mutators_list:set_h(h)
	row_item.mutators_list:set_bottom(self._info_bg_rect:h() - 2 * tweak_data.menu.info_padding)
	row_item.mutators_list:set_visible(mutators_active)
	
	local _, _, w, _ = row_item.mutators_title:text_rect()
	row_item.mutators_title:set_x(tweak_data.menu.info_padding)
	row_item.mutators_title:set_w(w)
	row_item.mutators_title:set_visible(mutators_active)
	
	row_item.mutators_title:set_top(row_item.mutators_list:top())
	row_item.mutators_list:set_left(row_item.mutators_title:right())
	
	row_item.mutators_title:set_position(math.round(row_item.mutators_title:x()), math.floor(row_item.mutators_title:y()))
	row_item.mutators_list:set_position(math.round(row_item.mutators_list:x()), math.floor(row_item.mutators_list:y()))
	
	row_item.stats_title:set_font_size(font_size)
	row_item.stats_text:set_font_size(font_size)
	local x, y, w, h = row_item.stats_title:text_rect()
	row_item.stats_title:set_x(tweak_data.menu.info_padding)
	row_item.stats_title:set_y(tweak_data.menu.info_padding + offset * 6)
	row_item.stats_title:set_w(w)
	row_item.stats_text:set_lefttop(row_item.stats_title:righttop())
	row_item.stats_text:set_w(row_item.gui_info_panel:w())
	row_item.stats_text:set_position(math.round(row_item.stats_text:x()), math.round(row_item.stats_text:y()))
	
	local _, _, _, h = row_item.heist_name:text_rect()
	local w = row_item.gui_info_panel:w()
	row_item.heist_name:set_height(h)
	row_item.heist_name:set_w(w)
	row_item.heist_briefing:set_w(w)
	row_item.heist_briefing:set_shape(row_item.heist_briefing:text_rect())
	row_item.heist_briefing:set_x(tweak_data.menu.info_padding)
	row_item.heist_briefing:set_y(tweak_data.menu.info_padding + offset * 7 + tweak_data.menu.info_padding * 2)
	row_item.heist_briefing:set_position(math.round(row_item.heist_briefing:x()), math.round(row_item.heist_briefing:y()))
end

function MenuNodeTableGui:mouse_pressed(button, x, y)
	--[[if self.item_panel:inside(x, y) and self._item_panel_parent:inside(x, y) and x > self:_mid_align() then
		if button == Idstring("mouse wheel down") then
			return self:wheel_scroll_start(-1)
		elseif button == Idstring("mouse wheel up") then
			return self:wheel_scroll_start(1)
		end
	end]]--
	MenuNodeTableGui.super.mouse_pressed(self, button, x, y)
	if button == Idstring("0") and self._mini_info_text:inside(x, y) then
		managers.network.account:overlay_activate("url", "http://store.steampowered.com/stats")
		return true
	end
end

function MenuNodeTableGui:mouse_moved(o, x, y)

	local inside = self._mini_info_text:inside(x, y)
	--self._mouse_over = inside
	return inside, inside and "link"
end